// --- [ rs232_int.c ] ---------------------------------------------------------

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <inttypes.h>
#include <avr/interrupt.h>
#include <stdlib.h>  
#include "rs232_int.h"

//#define RS232_INT_MODE

extern char msg[10];
extern uint8_t TWI_Buf[];
extern int32_t k[4];
extern int16_t s[3];
extern uint32_t sq;



#if RS232_MODE	== INTERRUPT
volatile uint8_t rs_buffer[TX_BUFFER_SIZE];
volatile uint8_t rs_tail = 0, rs_head = 0;
#endif

// ----------------------------------------------------------------------------
// 
void rs232_init(void)
{
/* ATMEGA8
UBRRL = TEILER;									// siehe rs232.h
UBRRH = 0;
UCSRB = (0<<RXCIE | 1<<TXEN | 0<<RXEN);
//UCSRC = (1<<URSEL | 0<<UMSEL) | (1<<UCSZ1) | (1<<UCSZ0);
// URSEL - Achtung bei ATmega 8
UCSRC = (0<<UMSEL) | (1<<UCSZ1) | (1<<UCSZ0);
*/


// ATMEGA88
UBRR0L = TEILER;									// siehe rs232_int.h
UBRR0H = 0;
UCSR0C = (0<<UMSEL01) | (1<<UCSZ01) | (1<<UCSZ00);
UCSR0B = TX_INT_DISABLE;

}

#if RS232_MODE == INTERRUPT
// ----------------------------------------------------------------------------
// ein einzelnes Zeichen auf COM ausgeben
void put_c(uint8_t c)
{

register uint8_t x;
x = rs_head;

// verhindern, dass der Puffer von hinten her berschrieben wird
// wartet, bis rs_tail durch Senden incrementiert wurde
while (((x + 1) & TX_BUFFER_MASK) == rs_tail);

rs_buffer[x++] = c;								// c in die Warteschlange einreihen
x &= TX_BUFFER_MASK;								// Pointer-berlauf maskieren
rs_head = x;										// Register zurckschreiben


if (UCSR0B == TX_INT_DISABLE)					// Prfen, ob TX disabled ist
	{
	x = rs_tail;									// Tail einlesen
	UCSR0B = TX_INT_ENABLE;						// TX enablen

	UDR0 = rs_buffer[x++];						// und ein Byte auf die Reise schicken
	x &= TX_BUFFER_MASK;							// Pointer incrementieren, maskieren
	rs_tail = x;									// und speichern
	}
}
#else

// ----------------------------------------------------------------------------
// ein einzelnes Zeichen auf COM ausgeben
void put_c(uint8_t c)
{
UDR0 = c;
while(!(UCSR0A & (1<<UDRE0)));
}

#endif

// ----------------------------------------------------------------------------
// einen String auf COM ausgeben
void put_s(char *s)
{

while(*s) put_c(*s++);

}

// ----------------------------------------------------------------------------
// gibt einen String aus dem Flash aus
void puts_p(const char *progmem_s)
{
register char c;
while ( (c = pgm_read_byte(progmem_s++)) )
	{
   put_c(c);
	}
}

#if RS232_MODE == INTERRUPT
// ----------------------------------------------------------------------------
// Nachdem ein Byte verschickt worden ist, wird der TXC-Interrufpt aufgerufen.
// Es wird geprft, ob weitere Daten im Ringpuffer sind.
// Wenn ja, dann wird das nchste Byte ausgegeben,
// wenn nicht, dann wird die bertragung beendet.
// Der Status von UCSR0B gibt zu erkennen, ob eine bertragung aktiv ist.
ISR (USART_TX_vect)
{

register uint8_t r;								// volatile rs_tail in ein Register laden
r = rs_tail;

if (!(r == rs_head))								// wenn rs_tail == rs_head, dann ist der Puffer leer
	{
	UDR0 = rs_buffer[r++];						// Byte in den Sender schreiben
	r &= TX_BUFFER_MASK;							// Pointer incrementieren und maskieren
	rs_tail = r;									// und zurckschreiben
	}
	
else													// der Puffer ist leer, bertragung beenden
	{
	UCSR0B = TX_INT_DISABLE;					// TX-Interrupt disablen
	}
}

#endif


/*
// ----------------------------------------------------------------------------
// wandelt Byte in Hexdarstellung um und gibt Hex auf COM aus
void print_hexbyte(unsigned char i)
{
unsigned char h;
    
h = i & 0xF0;              	// High nibble
h = h>>4;
h = h + '0';
    
if (h > '9') h += 7;

i &= 0x0F;         				// Low nibble
i +='0';
if (i > '9') i += 7;

put_c('0');
put_c('x');
put_c(h);
put_c(i);
put_c(' ');
}
*/


// --- [ eof ] ----------------------------------------------------------------